use crate::model::processing_input::processing_input;
use crate::model::check_correct_input::check_correct_input;
use crate::model::from_string_to_vec_bits::from_string_to_vec_bits;
use crate::model::hamming_code::hamming_code::hamming_decrypt_data;
use crate::model::hamming_code::hamming_code::hamming_encrypt_data;

#[path="utils/model_utils/processing_input.rs"]
mod processing_input;

#[path="utils/model_utils/check_correct_input.rs"]
mod check_correct_input;

#[path="utils/model_utils/from_string_to_vec_bits.rs"]
mod from_string_to_vec_bits;

#[path="utils/hamming_code_utils/hamming_code_seven_four.rs"]
mod hamming_code;

pub enum HammingMode{
    Encrypt,
    Decrypt
}

// ошибочная позиция 1 false true true
// ошибочная позиция 2 false false true
// ошибочная позиция 3 true false true
// ошибочная позиция 4 false true false
// ошибочная позиция 5 true true false
// ошибочная позиция 6 true false false
// ошибочная позиция 7 false false false

pub fn hamming(raw_input: String, mode: HammingMode) -> Result<String, String>{

    let length_of_code : usize = if let HammingMode::Encrypt = mode { 4 } else { 7 };

    let prepared_input : String = processing_input(&raw_input);

    let (fc, sc) = check_correct_input(&raw_input, &prepared_input, length_of_code);

    if !fc || !sc {

        Err("Ошибка. Проверьте корректность ввода.".to_string())

    } else {

        let mut data : String = String::new();

        let prepared_data: Vec<u8> = from_string_to_vec_bits(prepared_input);

        match mode {
            HammingMode::Encrypt => hamming_encrypt_data(&prepared_data, &mut data, length_of_code),
            HammingMode::Decrypt => hamming_decrypt_data(&prepared_data, &mut data, length_of_code),
        }

        return Ok(data);

    }
}

